/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * AQTTSPanel.java
 *
 * Created on Jul 28, 2011, 12:58:51 AM
 */
package org.robokind.ui.swing.speech;

import java.net.URISyntaxException;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.Session;
import org.apache.qpid.client.AMQConnectionFactory;
import org.apache.qpid.client.AMQQueue;
import org.osgi.framework.BundleContext;
import org.robokind.api.common.osgi.OSGiUtils;
import org.robokind.api.common.utils.Listener;
import org.robokind.api.speech.SpeechEventList;
import org.robokind.api.speech.SpeechService;
import org.robokind.api.speech.visemes.SpeechEventVisemeListener;
import org.robokind.api.speech.visemes.VisemeListener;
import org.robokind.api.speech.visemes.VisemeLogger;
import org.robokind.impl.speech.JMSSpeechService;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class AQTTSPanel extends javax.swing.JPanel {

    /** Creates new form AQTTSPanel */
    public AQTTSPanel() {
        initComponents();
    }

    public void setAQTTSFacade(JMSSpeechService speechService){
        mySpeechControl.setPlayable(speechService);
        myTTSPanel.setTTSEngine(speechService);
    }
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        myTTSPanel = new org.robokind.ui.swing.speech.TTSPanel();
        jPanel1 = new javax.swing.JPanel();
        mySpeechControl = new org.robokind.ui.swing.common.PlayControlPanel();
        btnConnect = new javax.swing.JButton();

        btnConnect.setText("Connect");
        btnConnect.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btnConnectActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addComponent(btnConnect)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(mySpeechControl, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(btnConnect, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 33, Short.MAX_VALUE)
                    .addComponent(mySpeechControl, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
            .addComponent(myTTSPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 351, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, 33, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(myTTSPanel, javax.swing.GroupLayout.DEFAULT_SIZE, 185, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void btnConnectActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btnConnectActionPerformed
        try{
            connect();
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }//GEN-LAST:event_btnConnectActionPerformed

    private void connect() throws Exception {
        String cmd = "speech.CommandQueue1; {create: always, node: {type: queue}}";
        String err = "speech.ErrorQueue1; {create: always, node: {type: queue}}";
        String speech = "speech.RequestQueue1; {create: always, node: {type: queue}}";
        String event = "speech.EventQueue1; {create: always, node: {type: queue}}";
        String conStr = "amqp://admin:admin@clientid/test?brokerlist='tcp://127.0.0.1:5672'";
        BundleContext context = OSGiUtils.getBundleContext(SpeechService.class);
        JMSSpeechService speechService = startServiceFrame(context, conStr, cmd, err, speech, event);
        mySpeechControl.setPlayable(speechService);
        myTTSPanel.setTTSEngine(speechService);
        
        VisemeListener visListener = new VisemeLogger();
        Listener<SpeechEventList> speechListener = new SpeechEventVisemeListener(visListener);
        speechService.addSpeechEventListener(speechListener);
    }
    
    public static JMSSpeechService startServiceFrame(BundleContext context,
            String conStr, String cmd, String err, String speech, String event) 
            throws JMSException, URISyntaxException, Exception {        
        ConnectionFactory cf = new AMQConnectionFactory(conStr);
        Connection connection = cf.createConnection();
        Destination reqDest = new AMQQueue(speech);
        Destination eventDest = new AMQQueue(event);
        connection.start();
        Session session = 
                connection.createSession(false, Session.CLIENT_ACKNOWLEDGE);
        Destination cmdDest = new AMQQueue(cmd);
        Destination errDest = new AMQQueue(err);
        
        JMSSpeechService service = new JMSSpeechService(
                session, cmdDest, errDest, reqDest, eventDest);
        return service;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btnConnect;
    private javax.swing.JPanel jPanel1;
    private org.robokind.ui.swing.common.PlayControlPanel mySpeechControl;
    private org.robokind.ui.swing.speech.TTSPanel myTTSPanel;
    // End of variables declaration//GEN-END:variables
}
