/*
 * Copyright 2011 Hanson Robokind LLC.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * VideoPanel.java
 *
 * Created on Jul 8, 2011, 8:42:02 PM
 */
package org.robokind.ui.swing.vision;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Image;
import java.beans.Beans;
import javax.swing.SwingUtilities;
import org.robokind.api.common.utils.Listener;
import org.robokind.api.vision.ImageEvent;
import org.robokind.api.vision.ImageRegion;
import org.robokind.api.vision.ImageRegionList;

/**
 *
 * @author Matthew Stevenson <www.robokind.org>
 */
public class VideoPanel extends javax.swing.JPanel{
    private Image myImage;
    private ImageRegionList myImageRegions;
    private Runnable myRepaint;
    private ImageCache myImageCache;
    private ImageRegionListCache myImageRegionListCache;
    
    /** Creates new form VideoPanel */
    public VideoPanel() {
        initComponents();
        if(Beans.isDesignTime()){
            return;
        }
        myRepaint = new Runnable() {
            @Override
            public void run() {
                repaint();
            }
        };
        myImageCache = new ImageCache();
        myImageRegionListCache = new ImageRegionListCache();
    }
    
    public Listener<ImageEvent> getImageEventListener(){
        return myImageCache;
    }
    
    public Listener<ImageRegionList> getImageRegionListListener(){
        return myImageRegionListCache;
    }
    
    @Override public void paint(Graphics g){
        if(myImage != null){
            g.drawImage(myImage, 0, 0, getWidth(), getHeight(), null);
        }
        if(myImageRegions != null && myImageRegions.getImageRegions() != null){
            g.setColor(Color.red);
            for(ImageRegion r : myImageRegions.getImageRegions()){
                int x = r.getX();
                int y = r.getY();
                int w = r.getWidth();
                int h = r.getHeight();
                g.drawRect(x, y, w, h);
            }
        }
    }
    

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
    class ImageCache implements Listener<ImageEvent>{        
        @Override
        public void handleEvent(ImageEvent event) {
            if(event == null){
                return;
            }
            Image img = event.getImage();
            if(img == null){
                return;
            }
            myImage = img;
            SwingUtilities.invokeLater(myRepaint);
        }
        
    }
    
    class ImageRegionListCache implements Listener<ImageRegionList>{        
        @Override
        public void handleEvent(ImageRegionList event) {
            if(event == null){
                return;
            }
            myImageRegions = event;
        }
        
    }
}
